@@ -195,3 +195,53 @@ importParallel <- local({
     res
   }
 })
+
+
+# Assert that 'Rscript --version' can be called and works
+#' @importFrom utils file_test
+assert_system_is_supported <- local({
+  results <- list()
+
+  ## utils::osVersion is only available in R (>= 3.5.0)
+  osVersion <- function() {
+    ns <- getNamespace("utils")
+    if (!exists("osVersion", envir = ns, inherits = TRUE)) {
+      osVersion <- get("osVersion", envir = ns, inherits = TRUE)
+      osVersion
+    } else {
+      "<unknown operating system>"
+    }
+  }
+  
+  function(method = "Rscript --version") {
+    method <- match.arg(method)
+    result <- results[[method]]
+    if (is.logical(result)) return(result)
+    
+    if (method == "Rscript --version") {
+      bin <- "Rscript"
+      if (.Platform[["OS.type"]] == "windows") bin <- sprintf("%s.exe", bin)
+      bin <- file.path(R.home("bin"), bin)
+      if (!file_test("-f", bin)) {
+        stop(sprintf("[INTERNAL ERROR]: %s:::assert_system_is_supported(method = \"%s\") failed, because file %s does not exists", .packageName, method, sQuote(bin)))
+      } else if (!file_test("-x", bin)) {
+        stop(sprintf("[INTERNAL ERROR]: %s:::assert_system_is_supported(method = \"%s\") failed, because %s is not an executable file", .packageName, method, sQuote(bin)))
+      }
+      
+      out <- system2(bin, args = "--version", stdout = TRUE, stderr = TRUE)
+      status <- attr(out, "status")
+      if (!is.null(status)) {
+        errmsg <- paste(c(attr(out, "errmsg"), ""), collapse = "")
+        stop(sprintf("The assertion test that system2(\"%s\", args = \"--version\", stdout = TRUE) works on your system (R %s on platform %s and %s) failed with a non-zero exit code (%s). It might be that your account or operating system does not allow this. The captured output was %s and the reported error was %s", bin, getRversion(), R.version$platform, osVersion(), status, sQuote(out), sQuote(errmsg)))
+      }
+
+      if (!grepl(getRversion(), out, fixed = TRUE)) {
+        stop(sprintf("[INTERNAL ERROR]: %s:::assert_system_is_supported(method = \"%s\") failed, because the output of %s does not contain the expected R version (%s): %s", .packageName, sQuote(method), sQuote(method), getRversion(), sQuote(out)))
+      }
+    }
+    
+    results[[method]] <<- TRUE
+    
+    TRUE
+  }
+})
